/* Dia -- an diagram creation/manipulation program
 * Copyright (C) 1998 Alexander Larsson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>

#include "intl.h"
#include "object.h"
#include "element.h"
#include "connectionpoint.h"
#include "render.h"
#include "attributes.h"
#include "widgets.h"
#include "properties.h"

#include "task.h"
#include "task_callbacks.h"
#include "mul.h"
#include "mul_callbacks.h"



//undo/redo functions
static MulState* mul_get_state(Mul *mul);
static void mul_set_state(Mul *mul, MulState *state);

//khorben callbacks
//mul properties
//open dialog
GtkWidget* mul_get_props(Mul *mul, Property *props, guint nprops)
{
	MulPropertiesDialog *prop_dialog;
	GtkWidget *dialog;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *border_color;
	GtkWidget *inner_color;
	GtkWidget *omenu;
//	GtkWidget *menu;
	GtkWidget *spin;
	GtkAdjustment *adj;

	if(mul->properties_dialog == NULL)
	{
		prop_dialog = g_new(MulPropertiesDialog, 1);
		mul->properties_dialog = prop_dialog;

		dialog = gtk_vbox_new(FALSE, 0);
		//what's this for??? :-( I gotta search for it
		gtk_object_ref(GTK_OBJECT(dialog));
		gtk_object_sink(GTK_OBJECT(dialog));
		prop_dialog->dialog = dialog;

		//option menu to choose task to work on
		hbox = gtk_hbox_new(FALSE, 5);
		label = gtk_label_new(_("Task"));
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		omenu = gtk_option_menu_new();
		prop_dialog->task_omenu = GTK_OPTION_MENU(omenu);
		mul_create_omenu(mul);
		mul->nulltask = mul->tasks->data;
		gtk_box_pack_start(GTK_BOX(hbox), omenu, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(dialog), hbox, FALSE, FALSE, 0);
		gtk_widget_show_all(hbox);

		//task to work on
		gtk_box_pack_start(GTK_BOX(dialog), task_get_props(mul->nulltask, props, nprops), FALSE, FALSE, 0);

		//product soze
		hbox = gtk_hbox_new(FALSE, 5);
		label = gtk_label_new(_("Product size"));
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		adj = (GtkAdjustment *)gtk_adjustment_new(g_slist_length(mul->tasks), 2.0, 256.0, 1.0, 5.0, 0.0);
		spin = gtk_spin_button_new(adj, 0, 0);
//		gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
		gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
		gtk_signal_connect(GTK_OBJECT(spin), "changed", mul_callback_spin, mul);
		gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(dialog), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);

		//color selectors
		hbox = gtk_hbox_new(TRUE, 5);
		label = gtk_label_new(_("Line colour"));
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		border_color = dia_color_selector_new();
		dia_color_selector_set_color((DiaColorSelector *)border_color, &mul->border_color);
		prop_dialog->fg_color = (DiaColorSelector *)border_color;
		gtk_box_pack_end(GTK_BOX(hbox), border_color, TRUE, TRUE,0);
		gtk_box_pack_start(GTK_BOX(dialog), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);

		hbox = gtk_hbox_new(TRUE, 5);
		label = gtk_label_new(_("Fill colour"));
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		inner_color = dia_color_selector_new();
		dia_color_selector_set_color((DiaColorSelector *)inner_color, &mul->inner_color);
		prop_dialog->bg_color = (DiaColorSelector *)inner_color;
		gtk_box_pack_end(GTK_BOX(hbox), inner_color, TRUE, TRUE,0);
		gtk_box_pack_start(GTK_BOX(dialog), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);
	}

	//update fields
	mul_fill_in_dialog(mul);
	gtk_widget_show(mul->properties_dialog->dialog);
	
	return mul->properties_dialog->dialog;
}

//fill in fields
void mul_fill_in_dialog(Mul *mul)
{
	MulPropertiesDialog *prop_dialog;

	prop_dialog = mul->properties_dialog;

	/* fill both tasks */
	task_fill_in_dialog(mul->nulltask);

	/* fill colour */
	dia_color_selector_set_color(prop_dialog->fg_color, &mul->border_color);
	dia_color_selector_set_color(prop_dialog->bg_color, &mul->inner_color);
}

//apply button callback
ObjectChange* mul_apply_props(Mul *mul)
	//FIXME return new_object_state_change
	//delete old menu
{
	ObjectState *old_state;
	MulPropertiesDialog *prop_dialog;
	GtkWidget *menuitem;
	int i;

	prop_dialog = mul->properties_dialog;
	old_state = (ObjectState*)mul_get_state(mul);

	task_apply_props(mul->nulltask);

	menuitem = gtk_menu_get_active(prop_dialog->task_menu);
	i = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
	mul_create_omenu(mul);

	if(i < g_slist_length(mul->tasks))
		gtk_option_menu_set_history(prop_dialog->task_omenu, i);

	dia_color_selector_get_color(prop_dialog->bg_color, &mul->inner_color);
	dia_color_selector_get_color(prop_dialog->fg_color, &mul->border_color);

	mul_update_data(mul);

	return new_object_state_change(&mul->element.object, old_state, (GetStateFunc)mul_get_state, (SetStateFunc)mul_set_state);
}

//create tasksel omenu
void mul_create_omenu(Mul *mul)
	//FIXME: destroy the old menu!!!
{
	int i;
	GtkWidget *menu;
	GtkWidget *menuitem;
	GSList *tasks;
	Task *task;
	GSList *group;

	menu = gtk_menu_new();
	mul->properties_dialog->task_menu = GTK_MENU(menu);
	group = NULL;
	tasks = mul->tasks;
	i = 0;
	do
	{
		task = tasks->data;
		menuitem = gtk_radio_menu_item_new_with_label(group, task->name);
		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", mul_callback_omenu, mul);
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(i));
		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(mul->properties_dialog->task_menu, menuitem);
		gtk_widget_show(menuitem);

		tasks = g_slist_next(tasks);
		i++;
	}
	while(tasks);

	gtk_option_menu_set_menu(mul->properties_dialog->task_omenu, menu);
}

//omenu callback
void mul_callback_omenu(GtkWidget *widget, gpointer data)
{
	Mul *mul;
	Task *task;

	mul = data;
	task = g_slist_nth_data(mul->tasks, GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget))));

	task->properties_dialog = mul->nulltask->properties_dialog;

	mul->nulltask = task;
	task_fill_in_dialog(task);

//	mul_fill_in_dialog(mul);
}

//spin callback
void mul_callback_spin(GtkWidget *widget, gpointer data)
{
	int i, n;
	Mul *mul;

	mul = data;
	i = g_slist_length(mul->tasks);
	n = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));

	if(i < n)
		while(i != n)
		{
			mul->tasks = g_slist_append(mul->tasks, task_create(NULL));
			i++;
		}
	else if(i > n)
		while(i != n)
		{
			task_destroy(g_slist_last(mul->tasks)->data);
			mul->tasks = g_slist_remove_link(mul->tasks, g_slist_last(mul->tasks));
			i--;
		}
//	else //i == n => do nothing
}

//undo/redo functions
//FIXME: task list for at least set_state!!!
static MulState* mul_get_state(Mul *mul)
{
	GSList *tasks;
	MulState *state = g_new0(MulState, 1);

	state->border_color = mul->border_color;
	state->inner_color = mul->inner_color;

	tasks = mul->tasks;
	do
	{
		state->tasks = g_slist_append(state->tasks, task_copy(tasks->data));
		tasks = tasks->next;
	}
	while(tasks);

	return state;
}

static void mul_set_state(Mul *mul, MulState *state)
{
	GSList *tasks;

	mul->border_color = state->border_color;
	mul->inner_color = state->inner_color;

	tasks = mul->tasks;
	do
	{
		task_destroy(tasks->data);
		tasks = g_slist_remove(tasks, tasks->data);
	}
	while(tasks);

	mul->tasks = NULL;
	tasks = state->tasks;
	do
	{
		mul->tasks = g_slist_append(mul->tasks , task_copy(tasks->data));
		tasks = g_slist_next(tasks);
	}
	while(tasks);
}
