/* Dia -- an diagram creation/manipulation program
 * Copyright (C) 1998 Alexander Larsson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>

#include "intl.h"
#include "object.h"
#include "element.h"
#include "connectionpoint.h"
#include "render.h"
#include "attributes.h"
#include "widgets.h"
#include "properties.h"


#include "task.h"
#include "task_callbacks.h"
#include "stask.h"
#include "stask_callbacks.h"



//undo/redo functions
static StaskState* stask_get_state(Stask *stask);
static void stask_set_state(Stask *stask, StaskState *state);


//khorben callbacks
//task properties
//open dialog
GtkWidget* stask_get_properties(Stask *stask, Property *props, guint nprops)
{
	StaskPropertiesDialog *prop_dialog;
	GtkWidget *dialog;

	if(stask->properties_dialog == NULL)
	{
		prop_dialog = g_new(StaskPropertiesDialog, 1);
		stask->properties_dialog = prop_dialog;

		dialog = gtk_vbox_new(FALSE, 0);
		//khorben: what's this for??? :-(
		gtk_object_ref(GTK_OBJECT(dialog));
		gtk_object_sink(GTK_OBJECT(dialog));
		prop_dialog->dialog = dialog;

		gtk_box_pack_start(GTK_BOX(dialog), task_get_props(stask->task, props, nprops), FALSE, FALSE, 0);
	}

	stask_fill_in_dialog(stask);
	gtk_widget_show(stask->properties_dialog->dialog);

	return stask->properties_dialog->dialog;
}

//fill in fields
void stask_fill_in_dialog(Stask *stask)
{
	StaskPropertiesDialog *prop_dialog;

	prop_dialog = stask->properties_dialog;

	task_fill_in_dialog(stask->task);
}

//apply button callback
ObjectChange* stask_apply_props(Stask *stask)
{
	ObjectState *old_state;
	StaskPropertiesDialog *prop_dialog;

	prop_dialog = stask->properties_dialog;
	old_state = (ObjectState*)stask_get_state(stask);

	task_apply_props(stask->task);

	stask_update_data(stask);

	return new_object_state_change(&stask->element.object, old_state, (GetStateFunc)stask_get_state, (SetStateFunc)stask_set_state);
}

//undo/redo functions
//FIXME: these two functions may destroy too much or not enough :(
static StaskState* stask_get_state(Stask *stask)
{
	StaskState *state = g_new0(StaskState, 1);

	state->task = task_copy(stask->task);

	return state;
}

static void stask_set_state(Stask *stask, StaskState *state)
{
	//task_destroy(stask->task);
	stask->task = task_copy(state->task);
}
