/* Dia -- an diagram creation/manipulation program
 * Copyright (C) 1998 Alexander Larsson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>

#include "intl.h"
#include "object.h"
#include "element.h"
#include "connectionpoint.h"
#include "render.h"
#include "attributes.h"
#include "widgets.h"
#include "properties.h"

#include "task.h"
#include "task_callbacks.h"
#include "sum.h"
#include "sum_callbacks.h"



char* sum_op_strings[] = { "==", "<>", "<", "<=", ">", ">=" };
char* sum_vp_strings[] = { "Value", "Parameter" };
//typedef enum { EQ, DIFF, LT, LET, GT, GET } sum_op_enum;

//misc callbacks
static void sum_props_set_sensitive(SumPropertiesDialog *prop_dialog, gboolean b);
//undo/redo functions
static SumState* sum_get_state(Sum *sum);
static void sum_set_state(Sum *sum, SumState *state);

//callbacks
/*static void sum_callback_op_toggle(GtkWidget *widget, gpointer data);
static void sum_callback_op_omenu1(GtkWidget *widget, gpointer data);
static void sum_callback_op_omenu2(GtkWidget *widget, gpointer data);
static void sum_callback_op_omenu3(GtkWidget *widget, gpointer data);
static void sum_callback_op_omenu4(GtkWidget *widget, gpointer data);
static void sum_callback_op_omenu5(GtkWidget *widget, gpointer data);
*/
//khorben callbacks
//sum properties
//open dialog
GtkWidget* sum_get_props(Sum *sum, Property *props, guint nprops)
{
	int i;
	SumPropertiesDialog *prop_dialog;
	GtkWidget *dialog;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *border_color;
	GtkWidget *inner_color;
//	GtkWidget *combo;
//	GList *popdown;
	GtkWidget *menu;
	GtkWidget *menuitem;
	GSList *group;
//	GtkWidget *radio;
//	GSList *rgroup;
	GtkWidget *separator;

	if(sum->properties_dialog == NULL)
	{
		prop_dialog = g_new(SumPropertiesDialog, 1);
		sum->properties_dialog = prop_dialog;

		dialog = gtk_vbox_new(FALSE, 0);
		//khorben: what's this for??? :-( I gotta search for it
		gtk_object_ref(GTK_OBJECT(dialog));
		gtk_object_sink(GTK_OBJECT(dialog));
		prop_dialog->dialog = dialog;

		gtk_box_pack_start(GTK_BOX(dialog), task_get_props(sum->task[0], props, nprops), FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(dialog), task_get_props(sum->task[1], props, nprops), FALSE, FALSE, 0);

		hbox = gtk_hbox_new(FALSE, 5);
		label = gtk_label_new(_("Determinist"));
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		prop_dialog->toggle = gtk_toggle_button_new();
		label = gtk_label_new(_("No"));
		gtk_container_add(GTK_CONTAINER(prop_dialog->toggle), label);
//		gtk_signal_connect(GTK_OBJECT(prop_dialog->toggle), "clicked", sum_callback_op_toggle, label);
		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->toggle, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(dialog), hbox, TRUE, TRUE, 5);
		gtk_widget_show_all(hbox);

		hbox = gtk_hbox_new(FALSE, 5);
		label = gtk_label_new(_("If"));
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);

		//task choice
		prop_dialog->omenu1 = gtk_option_menu_new();
		menu = gtk_menu_new();
		prop_dialog->menu1 = GTK_MENU(menu);
//		group = NULL;
		menuitem = gtk_menu_item_new_with_label(sum->task[0]->name);
//		menuitem = gtk_radio_menu_item_new_with_label(group, sum->task[0]->name);
//		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu1, sum);
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(0));
//		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		menuitem = gtk_menu_item_new_with_label(sum->task[1]->name);
//		menuitem = gtk_radio_menu_item_new_with_label(group, sum->task[1]->name);
//		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu1, sum);
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(1));
//		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_option_menu_set_menu(GTK_OPTION_MENU(prop_dialog->omenu1), menu);
		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->omenu1, FALSE, TRUE, 0);

		//vertical separator
		separator = gtk_vseparator_new();
		gtk_box_pack_start(GTK_BOX(hbox), separator, FALSE, FALSE, 0);
		gtk_widget_show(separator);

		//op
		prop_dialog->omenu2 = gtk_option_menu_new();
		menu = gtk_menu_new();
		prop_dialog->menu2 = GTK_MENU(menu);
//		group = NULL;
		//FIXME: dynamic instead of 6 could be good
		for(i = 0; i < 6; i++)
		{
			menuitem = gtk_menu_item_new_with_label(sum_op_strings[i]);
//			menuitem = gtk_radio_menu_item_new_with_label(group, sum_op_strings[i]);
//			gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu2, sum);
			gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(i));
//			group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
			gtk_menu_append(GTK_MENU(menu), menuitem);
			gtk_widget_show(menuitem);
		}
		gtk_option_menu_set_menu(GTK_OPTION_MENU(prop_dialog->omenu2), menu);
		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->omenu2, FALSE, FALSE, 0);

		//vertical separator
		separator = gtk_vseparator_new();
		gtk_box_pack_start(GTK_BOX(hbox), separator, FALSE, FALSE, 0);
		gtk_widget_show(separator);

		//choice between value or parameter
		prop_dialog->radio1 = gtk_radio_button_new(NULL);
//		rgroup = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio));
		prop_dialog->omenu3 = gtk_option_menu_new();
		menu = gtk_menu_new();
		prop_dialog->menu3 = GTK_MENU(menu);
		group = NULL;
		menuitem = gtk_menu_item_new_with_label(sum->task[0]->name);
//		menuitem = gtk_radio_menu_item_new_with_label(group, sum->task[0]->name);
//		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu3, sum);
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(0));
//		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		menuitem = gtk_menu_item_new_with_label(sum->task[1]->name);
//		menuitem = gtk_radio_menu_item_new_with_label(group, sum->task[1]->name);
//		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu3, sum);
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(1));
//		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_option_menu_set_menu(GTK_OPTION_MENU(prop_dialog->omenu3), menu);
//		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->omenu3, FALSE, TRUE, 0);
//		gtk_container_set_border_width(GTK_CONTAINER(prop_dialog->radio1), 0);
		gtk_container_add(GTK_CONTAINER(prop_dialog->radio1), prop_dialog->omenu3);
		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->radio1, FALSE, TRUE, 0);

		prop_dialog->radio2 = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(prop_dialog->radio1));
		prop_dialog->spin = gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(0.0, -32767.0, 32767.0, 0.1, 1.0, 0.0)), 1.0, 2);
		gtk_widget_set_usize(prop_dialog->spin, 75, 0);
		gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(prop_dialog->spin), TRUE);
/*		popdown = NULL;
		popdown = g_list_append(popdown, sum->task[0]->name);
		popdown = g_list_append(popdown, sum->task[1]->name);
		gtk_combo_set_popdown_strings(GTK_COMBO(prop_dialog->combo), popdown);
		*/
//		gtk_entry_set_text(GTK_ENTRY(prop_dialog->entry), "0");
//		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->entry, TRUE, TRUE, 0);
//		gtk_container_set_border_width(GTK_CONTAINER(prop_dialog->radio2), 0);
		gtk_container_add(GTK_CONTAINER(prop_dialog->radio2), prop_dialog->spin);
		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->radio2, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(dialog), hbox, TRUE, TRUE, 5);
		gtk_widget_show_all(hbox);

		hbox = gtk_hbox_new(FALSE, 5);
		label = gtk_label_new(_("Then"));
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		prop_dialog->omenu4 = gtk_option_menu_new();
		menu = gtk_menu_new();
		prop_dialog->menu4 = GTK_MENU(menu);
//		group = NULL;
		menuitem = gtk_menu_item_new_with_label(sum->task[0]->name);
//		menuitem = gtk_radio_menu_item_new_with_label(group, sum->task[0]->name);
//		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu4, sum);
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(0));
//		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		menuitem = gtk_menu_item_new_with_label(sum->task[1]->name);
//		menuitem = gtk_radio_menu_item_new_with_label(group, sum->task[1]->name);
//		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu4, sum);
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(1));
//		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_option_menu_set_menu(GTK_OPTION_MENU(prop_dialog->omenu4), menu);
		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->omenu4, TRUE, TRUE, 0);

		//vertical separator
		separator = gtk_vseparator_new();
		gtk_box_pack_start(GTK_BOX(hbox), separator, FALSE, FALSE, 0);
		gtk_widget_show(separator);

		label = gtk_label_new(_("Else"));
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		prop_dialog->omenu5 = gtk_option_menu_new();
		menu = gtk_menu_new();
		prop_dialog->menu5 = GTK_MENU(menu);
//		group = NULL;
		menuitem = gtk_menu_item_new_with_label(sum->task[0]->name);
//		menuitem = gtk_radio_menu_item_new_with_label(group, sum->task[0]->name);
//		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu5, sum);    
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(0));
//		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		menuitem = gtk_menu_item_new_with_label(sum->task[1]->name);             
//		menuitem = gtk_radio_menu_item_new_with_label(group, sum->task[1]->name);             
//		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sum_callback_op_omenu5, sum);
		gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(1));
//		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menuitem));
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_option_menu_set_menu(GTK_OPTION_MENU(prop_dialog->omenu5), menu);
		gtk_box_pack_start(GTK_BOX(hbox), prop_dialog->omenu5, TRUE, TRUE, 0); 
		gtk_box_pack_start(GTK_BOX(dialog), hbox, TRUE, TRUE, 5);
		gtk_widget_show_all(hbox);

		separator = gtk_hseparator_new();
		gtk_box_pack_start(GTK_BOX(dialog), separator, TRUE, TRUE, 5);
		gtk_widget_show(separator);

		hbox = gtk_hbox_new(TRUE, 5);
		label = gtk_label_new(_("Line colour"));
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		border_color = dia_color_selector_new();
		dia_color_selector_set_color((DiaColorSelector *)border_color, &sum->border_color);
		prop_dialog->fg_color = (DiaColorSelector *)border_color;
		gtk_box_pack_end(GTK_BOX(hbox), border_color, TRUE, TRUE,0);
		gtk_box_pack_start(GTK_BOX(dialog), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);

		hbox = gtk_hbox_new(TRUE, 5);
		label = gtk_label_new(_("Fill colour"));
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		inner_color = dia_color_selector_new();
		dia_color_selector_set_color((DiaColorSelector *)inner_color, &sum->inner_color);
		prop_dialog->bg_color = (DiaColorSelector *)inner_color;
		gtk_box_pack_end(GTK_BOX(hbox), inner_color, TRUE, TRUE,0);
		gtk_box_pack_start(GTK_BOX(dialog), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);
	}
	
	sum_fill_in_dialog(sum);
	gtk_widget_show(sum->properties_dialog->dialog);
	
	return sum->properties_dialog->dialog;
}

//fill in fields
void sum_fill_in_dialog(Sum *sum)
{
	SumPropertiesDialog *prop_dialog;

	prop_dialog = sum->properties_dialog;

	/* fill both tasks */
	task_fill_in_dialog(sum->task[0]);
	task_fill_in_dialog(sum->task[1]);

	/* determinist */
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dialog->toggle), sum->determinist);
	gtk_option_menu_set_history(GTK_OPTION_MENU(prop_dialog->omenu1), sum->tif1 == sum->task[1]);
	gtk_option_menu_set_history(GTK_OPTION_MENU(prop_dialog->omenu2), sum->op);
	gtk_option_menu_set_history(GTK_OPTION_MENU(prop_dialog->omenu3), sum->tif2 == sum->task[1]);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dialog->radio1), !sum->value);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dialog->radio2), sum->value);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(prop_dialog->spin), sum->rif2);
	gtk_option_menu_set_history(GTK_OPTION_MENU(prop_dialog->omenu4), sum->tthen == sum->task[1]);
	gtk_option_menu_set_history(GTK_OPTION_MENU(prop_dialog->omenu5), sum->telse == sum->task[1]);

	sum_props_set_sensitive(prop_dialog, sum->determinist);

	/* fill colour */
	dia_color_selector_set_color(prop_dialog->fg_color, &sum->border_color);
	dia_color_selector_set_color(prop_dialog->bg_color, &sum->inner_color);
}

//apply button callback
ObjectChange* sum_apply_props(Sum *sum)
{
	ObjectState *old_state;
	SumPropertiesDialog *prop_dialog;
	GtkWidget *menuitem;

	prop_dialog = sum->properties_dialog;
	old_state = (ObjectState*)sum_get_state(sum);

	task_apply_props(sum->task[0]);
	task_apply_props(sum->task[1]);

	sum->determinist = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prop_dialog->toggle));

	menuitem = gtk_menu_get_active(prop_dialog->menu1);
	sum->tif1 = sum->task[GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)))];

	menuitem = gtk_menu_get_active(prop_dialog->menu2);
	sum->op = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));

	menuitem = gtk_menu_get_active(prop_dialog->menu3);
	sum->tif2 = sum->task[GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)))];

	sum->value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prop_dialog->radio2));

	sum->rif2 = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(prop_dialog->spin));

	menuitem = gtk_menu_get_active(prop_dialog->menu4);
	sum->tthen = sum->task[GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)))];

	menuitem = gtk_menu_get_active(prop_dialog->menu5);
	sum->telse = sum->task[GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)))];

	sum_props_set_sensitive(prop_dialog, sum->determinist);

	dia_color_selector_get_color(prop_dialog->bg_color, &sum->inner_color);
	dia_color_selector_get_color(prop_dialog->fg_color, &sum->border_color);

	sum_update_data(sum);

	return new_object_state_change(&sum->element.object, old_state, (GetStateFunc)sum_get_state, (SetStateFunc)sum_set_state);
}

static void sum_props_set_sensitive(SumPropertiesDialog *prop_dialog, gboolean b)
{
	gtk_widget_set_sensitive(prop_dialog->omenu1, b);
	gtk_widget_set_sensitive(prop_dialog->omenu2, b);
	gtk_widget_set_sensitive(prop_dialog->radio1, b);
	gtk_widget_set_sensitive(prop_dialog->omenu3, b);
	gtk_widget_set_sensitive(prop_dialog->radio2, b);
	gtk_widget_set_sensitive(prop_dialog->spin, b);
	gtk_widget_set_sensitive(prop_dialog->omenu4, b);
	gtk_widget_set_sensitive(prop_dialog->omenu5, b);
}

//undo/redo functions
static SumState* sum_get_state(Sum *sum)
{
	SumState *state = g_new0(SumState, 1);

	state->border_color = sum->border_color;
	state->inner_color = sum->inner_color;

	//FIXME
	state->font = font_getfont(_("Courier"));

	state->task[0] = task_copy(sum->task[0]);
	state->task[1] = task_copy(sum->task[1]);

	state->determinist = sum->determinist;
	state->tif1 = state->task[sum->tif1 == sum->task[1]];
	state->op = sum->op;
	state->tif2 = state->task[sum->tif2 == sum->task[1]];
	state->value = sum->value;
	state->rif2 = sum->rif2;
	state->tthen = state->task[sum->tthen == sum->task[1]];
	state->telse = state->task[sum->telse == sum->task[1]];

	return state;
}

static void sum_set_state(Sum *sum, SumState *state)
{
	sum->border_color = state->border_color;
	sum->inner_color = state->inner_color;

	sum->font = font_getfont(_("Courier"));

	sum->task[0] = task_copy(state->task[0]);
	sum->task[1] = task_copy(state->task[1]);

	sum->determinist = state->determinist;
	sum->tif1 = sum->task[state->tif1 == state->task[1]];
	sum->op = state->op;
	sum->tif2 = sum->task[state->tif2 == state->task[1]];
	sum->value = state->value;
	sum->rif2 = state->rif2;
	sum->tthen = sum->task[state->tthen == state->task[1]];
	sum->telse = sum->task[state->telse == state->task[1]];
}


//callbacks
/*static void sum_callback_op_toggle(GtkWidget *widget, gpointer data)
{
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
		gtk_label_set_text(GTK_LABEL(data), _("Yes"));
	else
		gtk_label_set_text(GTK_LABEL(data), _("No"));
}

static void sum_callback_op_omenu1(GtkWidget *widget, gpointer data)
{
	Sum *sum = data;

	sum->tif1 = sum->task[GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)))];
}

static void sum_callback_op_omenu2(GtkWidget *widget, gpointer data)
{
	Sum *sum = data;

	sum->op = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)));
}

static void sum_callback_op_omenu3(GtkWidget *widget, gpointer data)
{
	Sum *sum = data;

	sum->tif2 = sum->task[GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)))];
}

static void sum_callback_op_omenu4(GtkWidget *widget, gpointer data)
{
	Sum *sum = data;

	sum->tthen = sum->task[GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)))];
}

static void sum_callback_op_omenu5(GtkWidget *widget, gpointer data)
{
	Sum *sum = data;

	sum->telse = sum->task[GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)))];
}*/
