/* Dia -- an diagram creation/manipulation program
 * Copyright (C) 1998 Alexander Larsson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>

#include "intl.h"
#include "object.h"
#include "element.h"
#include "connectionpoint.h"
#include "render.h"
#include "attributes.h"
#include "widgets.h"
#include "properties.h"

#include "task.h"
#include "task_callbacks.h"



//misc callbacks
//modules callbacks
//open window
static void task_callback_modules(GtkWidget* widget, gpointer data);
//fill in fields
static void task_fill_in_modules_window(Task *task);
//close window
static void task_callback_modules_window_close(GtkWidget* window, gpointer data);
//apply button callback
static void task_modules_apply_props(GtkWidget* widget, gpointer data);
//usescorba toggle button
static void task_callback_modules_toggle(GtkWidget* widget, gpointer data);
//sensitive widgets
static void task_modules_set_sensitive(TaskModulesWindow* mod_window, gboolean b);
//entry callback
//static void task_entry_callback(GtkWidget* widget, gpointer data);

//khorben callbacks
//task properties
//open dialog
GtkWidget* task_get_props(Task *task, Property *props, guint nprops)
{
	TaskPropertiesDialog *prop_dialog;
	GtkWidget *dialog;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *omenu;
	GtkWidget *menu;
	GSList *group;
	GtkWidget *submenu;
	GtkWidget *menuitem;
	GtkWidget *border_color;
	GtkWidget *inner_color;
	GtkWidget *frame;
	GtkWidget *bbox;
	GtkWidget *button;

	prop_dialog = g_new(TaskPropertiesDialog, 1);
	task->properties_dialog = prop_dialog;

	dialog = gtk_frame_new(_("Task properties"));
	prop_dialog->dialog = dialog;

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(dialog), vbox);
	gtk_widget_show_all(dialog);

	/* name entry */
	hbox = gtk_hbox_new(FALSE, 5);
	label = gtk_label_new(_("Name"));
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
	entry = gtk_entry_new();
	//FIXME: this could be more ser friendly
	//but I dunno how to update the bounding box
	//gtk_signal_connect(GTK_OBJECT(entry), "activate", task_entry_callback, task);
	prop_dialog->name = GTK_ENTRY(entry);
	gtk_box_pack_end(GTK_BOX (hbox), entry, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show_all(hbox);

	/* type entry */
	hbox = gtk_hbox_new(FALSE, 5);
	label = gtk_label_new(_("Type"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);

	omenu = gtk_option_menu_new();
	menu = gtk_menu_new ();
	prop_dialog->type_menu = GTK_MENU(menu);
	prop_dialog->type_omenu = GTK_OPTION_MENU(omenu);
	submenu = NULL;
	group = NULL;

	menuitem = gtk_radio_menu_item_new_with_label (group, _("Primitive"));
	gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(TASK_PRIMITIVE));
	group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
	gtk_menu_append(GTK_MENU (menu), menuitem);
	gtk_widget_show(menuitem);

	menuitem = gtk_radio_menu_item_new_with_label (group, _("Composite"));
	gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(TASK_COMPOSITE));
	group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
	gtk_menu_append(GTK_MENU (menu), menuitem);
	gtk_widget_show(menuitem);

	gtk_option_menu_set_menu(GTK_OPTION_MENU (omenu), menu);
	gtk_box_pack_end (GTK_BOX(hbox), omenu, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show_all(hbox);

	hbox = gtk_hbox_new(TRUE, 5);
	label = gtk_label_new(_("Line colour"));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
	border_color = dia_color_selector_new();
	dia_color_selector_set_color((DiaColorSelector *)border_color, &task->border_color);
	prop_dialog->fg_color = (DiaColorSelector *)border_color;
	gtk_box_pack_end(GTK_BOX(hbox), border_color, TRUE, TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show_all(hbox);

	hbox = gtk_hbox_new(TRUE, 5);
	label = gtk_label_new(_("Fill colour"));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
	inner_color = dia_color_selector_new();
	dia_color_selector_set_color((DiaColorSelector *)inner_color, &task->inner_color);
	prop_dialog->bg_color = (DiaColorSelector *)inner_color;
	gtk_box_pack_end(GTK_BOX(hbox), inner_color, TRUE, TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show_all(hbox);

	/* parameters */
	frame = gtk_frame_new(_("Parameters"));
	bbox = gtk_hbutton_box_new();
	gtk_container_set_border_width(GTK_CONTAINER (bbox), 5);
	gtk_container_add(GTK_CONTAINER (frame), bbox);
	button = gtk_button_new_with_label(_("Input"));
	gtk_container_add(GTK_CONTAINER(bbox), button);
	button = gtk_button_new_with_label(_("Input Output"));
	gtk_container_add(GTK_CONTAINER(bbox), button);
	button = gtk_button_new_with_label(_("Output"));
	gtk_container_add(GTK_CONTAINER(bbox), button);
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 5);
	gtk_widget_show_all(frame);

	/* module definitions button */
	hbox = gtk_hbox_new(FALSE, 5);
	prop_dialog->modules_button = gtk_button_new_with_label(_("Module Definitions"));
	gtk_signal_connect(GTK_OBJECT(prop_dialog->modules_button), "clicked", task_callback_modules, NULL);
	gtk_object_set_user_data(GTK_OBJECT(prop_dialog->modules_button), task);
	gtk_box_pack_end(GTK_BOX(hbox), prop_dialog->modules_button, FALSE, TRUE, 0);
	gtk_box_pack_end(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
	gtk_widget_show_all(hbox);

	return dialog;
}

/* apply button callback */
void task_apply_props(Task *task)
{
	TaskPropertiesDialog *prop_dialog;
	char *str;
	GtkWidget *menuitem;

	prop_dialog = task->properties_dialog;

	/* read from dialog and put in object: */
	g_free(task->name);

	str = gtk_entry_get_text(prop_dialog->name);
	if(strlen(str) != 0)
		task->name = g_strdup(str);
	else
		task->name = NULL;

	menuitem = gtk_menu_get_active(prop_dialog->type_menu);
	task->type = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));

	dia_color_selector_get_color(prop_dialog->bg_color, &task->inner_color);
	dia_color_selector_get_color(prop_dialog->fg_color, &task->border_color);

	 gtk_widget_set_sensitive(prop_dialog->modules_button, (task->type == TASK_PRIMITIVE));
	if(!task->type == TASK_PRIMITIVE && task->modules_window != NULL)
		gtk_widget_hide(task->modules_window->window);
}

/* fill in fields */
void task_fill_in_dialog(Task *task)
{
	TaskPropertiesDialog *prop_dialog;

	prop_dialog = task->properties_dialog;

	if(task->name != NULL)
		gtk_entry_set_text(prop_dialog->name, task->name);
	else
		gtk_entry_set_text(prop_dialog->name, "");

	gtk_option_menu_set_history(prop_dialog->type_omenu, task->type);
	gtk_widget_set_sensitive(prop_dialog->modules_button, (task->type == TASK_PRIMITIVE));

	dia_color_selector_set_color(prop_dialog->fg_color, &task->border_color);
	dia_color_selector_set_color(prop_dialog->bg_color, &task->inner_color);

	gtk_object_set_user_data(GTK_OBJECT(prop_dialog->modules_button), task);
}

//modules window
//open window
static void task_callback_modules(GtkWidget* widget, gpointer data)
{
	char *str;
	Task* task = gtk_object_get_user_data(GTK_OBJECT(widget));

	str = g_malloc(strlen(_("Module definitions")) + strlen(task->name) + 3);
	sprintf(str, "%s: %s", _("Module definitions"), task->name);

	if(task->modules_window == NULL)
	{
		TaskModulesWindow *mod_window;
		GtkWidget* vbox;
		GtkWidget* hbox;
		GtkWidget* frame;
		GtkWidget* bbox;
		GtkWidget* label;
		GtkWidget* entry;
		GtkWidget* separator;
		GtkWidget* button;

		mod_window = g_new(TaskModulesWindow, 1);
		task->modules_window = mod_window;

		mod_window->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(mod_window->window), str);
		gtk_signal_connect(GTK_OBJECT(mod_window->window), "delete_event", gtk_widget_hide, NULL);

		vbox = gtk_vbox_new(FALSE, 0);
		hbox = gtk_hbox_new(FALSE, 5);
		label = gtk_label_new(_("Uses CORBA"));
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		mod_window->label = gtk_label_new(_("No"));
		mod_window->toggle = gtk_toggle_button_new();
		gtk_container_add(GTK_CONTAINER(mod_window->toggle), mod_window->label);
		gtk_signal_connect(GTK_OBJECT(mod_window->toggle), "clicked", task_callback_modules_toggle, mod_window->label);
		gtk_box_pack_start(GTK_BOX(hbox), mod_window->toggle, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);

		hbox = gtk_hbox_new(FALSE, 5);
		label = gtk_label_new(_("Server name"));
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		entry = gtk_entry_new();
		mod_window->servername = GTK_ENTRY(entry);
		gtk_box_pack_end(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);

		separator = gtk_hseparator_new();
		gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);
		gtk_widget_show(separator);

		hbox = gtk_hbox_new(FALSE, 5);
		mod_window->labelo = gtk_label_new(_("CORBA Object name"));
		gtk_box_pack_start(GTK_BOX(hbox), mod_window->labelo, FALSE, TRUE, 0);
		mod_window->corbaobjectname = gtk_entry_new();
//		mod_window->corbaobjectname = GTK_ENTRY(entry);
		gtk_box_pack_end(GTK_BOX(hbox), mod_window->corbaobjectname, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);

		hbox = gtk_hbox_new(FALSE, 5);
		mod_window->labelm = gtk_label_new(_("CORBA Method name"));
		gtk_box_pack_start(GTK_BOX(hbox), mod_window->labelm, FALSE, TRUE, 0);
		mod_window->corbamethodname = gtk_entry_new();
//		mod_window->corbamethodname = GTK_ENTRY(entry);
		gtk_box_pack_end(GTK_BOX(hbox), mod_window->corbamethodname, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
		gtk_widget_show_all(hbox);

		frame = gtk_frame_new(_("Files"));
		gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
		bbox = gtk_hbutton_box_new();
		gtk_container_set_border_width(GTK_CONTAINER(bbox), 5);
		gtk_container_add(GTK_CONTAINER(frame), bbox);
		button = gtk_button_new_with_label(_("Input"));
		gtk_container_add(GTK_CONTAINER(bbox), button);
		button = gtk_button_new_with_label(_("Input Output"));
		gtk_container_add(GTK_CONTAINER(bbox), button);
		button = gtk_button_new_with_label(_("Output"));
		gtk_container_add(GTK_CONTAINER(bbox), button);
		gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
		gtk_widget_show_all(frame);

		hbox = gtk_hbox_new(TRUE, 20);
		mod_window->button = gtk_button_new_with_label(_("Method Parameters"));
		gtk_box_pack_start(GTK_BOX(hbox), mod_window->button, TRUE, TRUE, 20);
		mod_window->check = gtk_check_button_new_with_label(_("PaCO module"));
		gtk_box_pack_start(GTK_BOX(hbox), mod_window->check, TRUE, TRUE, 20);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
		gtk_widget_show_all(hbox);

		separator = gtk_hseparator_new();
		gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 5);
		gtk_widget_show(separator);

		hbox = gtk_hbox_new(FALSE, 10);
		button = gtk_button_new_with_label(_("Close"));
		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
		gtk_widget_set_usize(button, 100, 0);
		gtk_signal_connect(GTK_OBJECT(button), "clicked", task_callback_modules_window_close, task->modules_window->window);
		gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 10);

		button = gtk_button_new_with_label(_("Apply"));
		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
		gtk_widget_set_usize(button, 100, 0);
		gtk_object_set_user_data(GTK_OBJECT(button), task);
		gtk_signal_connect(GTK_OBJECT(button), "clicked", task_modules_apply_props, NULL);
		gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 10);

		gtk_box_pack_end(GTK_BOX(vbox), hbox, TRUE, TRUE, 10);
		gtk_widget_show_all(hbox);

		gtk_container_add(GTK_CONTAINER(mod_window->window), vbox);
		gtk_widget_show(vbox);
	}
	else
		gtk_window_set_title(GTK_WINDOW(task->modules_window->window), str);

	gtk_widget_show(task->modules_window->window);

	g_free(str);
	task_fill_in_modules_window(task);
}

//fill in fields
static void task_fill_in_modules_window(Task *task)
{
	TaskModulesWindow *mod_window;

	mod_window = task->modules_window;

	if(task->servername != NULL)
		gtk_entry_set_text(mod_window->servername, task->servername);
	else
		gtk_entry_set_text(mod_window->servername, "");

	if(task->corbaobjectname != NULL)
		gtk_entry_set_text(GTK_ENTRY(mod_window->corbaobjectname), task->corbaobjectname);
	else
		gtk_entry_set_text(GTK_ENTRY(mod_window->corbaobjectname), "");

	if(task->corbamethodname != NULL)
		gtk_entry_set_text(GTK_ENTRY(mod_window->corbamethodname), task->corbamethodname);
	else
		gtk_entry_set_text(GTK_ENTRY(mod_window->corbamethodname), "");

	//FIXME: booleans seems to be equal in Gtk and Dia
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mod_window->check), task->usespaco);

	task_modules_set_sensitive(mod_window, task->usescorba);
}

//close window
static void task_callback_modules_window_close(GtkWidget* widget, gpointer data)
{
	gtk_widget_hide(data);
}

static void task_callback_modules_toggle(GtkWidget* widget, gpointer data)
{
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
		gtk_label_set_text(GTK_LABEL(data), _("Yes"));
	else
		gtk_label_set_text(GTK_LABEL(data), _("No"));
}

//apply button callback
static void task_modules_apply_props(GtkWidget* widget, gpointer data)
{
	TaskModulesWindow *mod_window;
	char *str;
	Task* task = gtk_object_get_user_data(GTK_OBJECT(widget));

	mod_window = task->modules_window;

	g_free(task->servername);
	str = gtk_entry_get_text(mod_window->servername);
	if(strlen(str) != 0)
		task->servername = g_strdup(str);
	else
		task->servername = NULL;

	task->usescorba = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mod_window->toggle));

	g_free(task->corbaobjectname);
	str = gtk_entry_get_text(GTK_ENTRY(mod_window->corbaobjectname));
	if(strlen(str) != 0)
		task->corbaobjectname = g_strdup(str);
	else
		task->corbaobjectname = NULL;

	g_free(task->corbamethodname);
	str = gtk_entry_get_text(GTK_ENTRY(mod_window->corbamethodname));
	if(strlen(str) != 0)
		task->corbamethodname = g_strdup(str);
	else
		task->corbamethodname = NULL;

	task->usespaco = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mod_window->check));

	task_modules_set_sensitive(mod_window, task->usescorba);

	//FIXME: this needed?
//	task_update_data(task);
}

//misc modules
//sensitive widgets
static void task_modules_set_sensitive(TaskModulesWindow* mod_window, gboolean b)
{
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mod_window->toggle), b);
	gtk_widget_set_sensitive(mod_window->corbaobjectname, b);
	gtk_widget_set_sensitive(mod_window->corbamethodname, b);
	gtk_widget_set_sensitive(mod_window->labelo, b);
	gtk_widget_set_sensitive(mod_window->labelm, b);
	gtk_widget_set_sensitive(mod_window->button, b);
	gtk_widget_set_sensitive(mod_window->check, b);
}

//misc callbacks
//entry callback
/*static void task_entry_callback(GtkWidget* widget, gpointer data)
{
	task_apply_props(data);
}*/
